#include "SRBoomSprite.h"
#include "SRGameObject.h"
#include "SRGraphicsUtils.h"

CGImageRef SRBoomSprite::fBoomPicture = 0;

SRBoomSprite::SRBoomSprite( SRObject *creator )
	: fCurrentFrame(0)
{
    if( fBoomPicture == 0 )
    {
        fBoomPicture = SRGraphicsUtils::CreateCGImageForResourceNamed(CFSTR("boom"));
    }
    
	fImage = fBoomPicture;
	
    fWidth = (creator->width() < kMinWidth) ? kMinWidth : creator->width();
    fHeight = (creator->height() < kMinHeight) ? kMinHeight : creator->height();
    
    fX = creator->xPosition();
    fY = creator->yPosition();
    
    fVelocityX = creator->xVelocity();
    fVelocityY = creator->yVelocity();
    
    fXSlowdownFactor = -1;
    fYSlowdownFactor = -1;
    
    if(fVelocityX < 0)
    {
        fXSlowdownFactor = 1;
    }
    
    if(fVelocityY < 0)
    {
        fYSlowdownFactor = 1;
    }
    
    fAccelX = kSlowdown * fXSlowdownFactor;
    fAccelY = kSlowdown * fYSlowdownFactor;
}

SRBoomSprite::~SRBoomSprite()
{
}

void SRBoomSprite::Think()
{
    if(fCurrentFrame >= 0 && fCurrentFrame <= 10)
    {
        fWidth *= kGrowthFactor;
        fHeight *= kGrowthFactor;
    }
    else if(fCurrentFrame > 10 && fCurrentFrame < 20)
    {
        fWidth /= kGrowthFactor;
        fHeight /= kGrowthFactor;
    }
    else
    {
        SRGameObject::instance->Recycle(this);
    }
    
    if( (fVelocityX*fXSlowdownFactor) >= 0 )
    {
        fVelocityX = 0;
        fAccelX = 0;
    }
    
    if( (fVelocityY*fYSlowdownFactor) >= 0 )
    {
        fVelocityY = 0;
        fAccelY = 0;
    }
    
    fCurrentFrame++;
}

void SRBoomSprite::Draw( CGContextRef theRef )
{
	SRRect bounds( fX - (fWidth/2), fY - (fHeight/2), fWidth, fHeight );

	CGContextDrawImage( theRef, bounds, fImage );
}
