/*
 *  SRGraphicsUtils.cpp
 *  SpaceRatXCode
 *
 *  Created by James Dessart on 09/08/06.
 *  Copyright 2006 James Dessart. All rights reserved.
 *
 */

#include "SRGraphicsUtils.h"

SRRect::SRRect( float x, float y, float width, float height )
	: fX(x), fY(y), fWidth(width), fHeight(height)
{
}

SRRect::SRRect( Rect &theRect )
{
	*this = theRect;
}

SRRect::SRRect( CGRect &theRect )
{
	*this = theRect;
}

SRRect::operator CGRect()
{
	return CGRectMake( fX, fY, fWidth, fHeight );
}

SRRect::operator Rect()
{
	Rect theRect = {fX,fY, fX+fWidth - 1, fY+fHeight - 1};
	
	return theRect;
}

SRRect &
SRRect::operator =( CGRect &inRect )
{
	fX = inRect.origin.x;
	fY = inRect.origin.y;
	
	fWidth = inRect.size.width;
	fHeight = inRect.size.height;
	
	return (*this);
}

SRRect &
SRRect::operator =( Rect &inRect )
{
	fX = inRect.left;
	fY = inRect.top;
	
	fWidth = inRect.right - inRect.left + 1;
	fHeight = inRect.bottom - inRect.top + 1;
	
	return (*this);
}

#pragma mark

static const float kUnsignedShortMaxAsFloat = 65535.0f;

inline float colour_short_to_float( unsigned short inShort )
{
	return float(inShort) / kUnsignedShortMaxAsFloat;
}

SRRgbColour::SRRgbColour() : fRed(0.0f), fGreen(0.0f), fBlue(0.0f), fAlpha(0.0f)
{
}

SRRgbColour::SRRgbColour( RGBColor &inColor )
{
	*this = inColor;
}

SRRgbColour &
SRRgbColour::operator =( RGBColor &inColor )
{
	fRed = colour_short_to_float(inColor.red);
	fGreen = colour_short_to_float(inColor.green);
	fBlue = colour_short_to_float(inColor.blue);
	fAlpha = 1.0f;
	
	return (*this);
}

SRRgbColour::SRRgbColour( float inRed, float inGreen, float inBlue, float inAlpha )
	: fRed(inRed), fGreen(inGreen), fBlue(inBlue), fAlpha(inAlpha)
{
}

SRRgbColour::operator RGBColor()
{
	RGBColor theColor = { fRed * kUnsignedShortMaxAsFloat,
						  fGreen * kUnsignedShortMaxAsFloat,
						  fBlue *kUnsignedShortMaxAsFloat };
	return theColor;
}

float
SRRgbColour::red()
{
	return fRed;
}

float
SRRgbColour::green()
{
	return fGreen;
}

float
SRRgbColour::blue()
{
	return fBlue;
}

float
SRRgbColour::alpha()
{
	return fAlpha;
}

#pragma mark

bool SRGraphicsUtils::UsingQuickdraw = 0;

CGImageRef
SRGraphicsUtils::CreateCGImageForResourceNamed( CFStringRef theName )
{
	CGImageRef theImage;

	// We'll assume png images
	CFURLRef theImageURL = CFBundleCopyResourceURL( CFBundleGetMainBundle(),
													theName,
													CFSTR("png"),
													0 );
	
	if( theImageURL == 0 ) return 0;
	
	CGDataProviderRef theProvider = CGDataProviderCreateWithURL(theImageURL);
	
	CFRelease(theImageURL);
	
	if( theProvider == 0 ) return 0;
	
	theImage = CGImageCreateWithPNGDataProvider( theProvider, 0, true, kCGRenderingIntentDefault );
	
	CFRelease(theProvider);
	
	return theImage;

// This was my original implementation, but it's dependant on functionality in 10.4!
// The new implementation, above, only depends on 10.2, a more reasonable requirement.	
//	CGImageSourceRef theImageSource
//		= CGImageSourceCreateWithURL( theImageURL, 0 );
//	
//	CFRelease(theImageURL);
//	
//	if( theImageSource == 0 ) return 0;
//	
//	// Grab the first image, since we only expect one image
//	CGImageRef theImage = CGImageSourceCreateImageAtIndex( theImageSource, 0, 0 );
//	
//	CFRelease(theImageSource);
}

void
SRGraphicsUtils::CreateCGImagesForFramedResourcesNamed
	( CFStringRef theBasename, CGImageRef *outImages, int inFrameCount )
{
	CFStringRef currentName;

	int i;
	for( i = 0; i < inFrameCount; i++ )
	{
		currentName = CFStringCreateWithFormat( kCFAllocatorDefault, 0,
												CFSTR("%@_frame%d"),
												theBasename, i );
		outImages[i] = CreateCGImageForResourceNamed( currentName );
		
		CFRelease(currentName);
	}
}
