#include "SRPlayerSprite.h"
#include "SRAmmoSprite.h"
#include "SRGameObject.h"
#include "WindowDefs.h"
#include "SRSoundManager.h"
#include "SRGraphicsUtils.h"

//#include <iostream>

SRPlayerSprite *SRPlayerSprite::fInstance = 0;

SRPlayerSprite::SRPlayerSprite() : fDirection( DirectionHalt ),
	fAmmoTimer(0), fFireNow(false)
{
    fInstance = this;
    
    ResetPosition();
    
    // Load the ship's icons
	fLeftImage = SRGraphicsUtils::CreateCGImageForResourceNamed(CFSTR("ship_left"));
	fRightImage = SRGraphicsUtils::CreateCGImageForResourceNamed(CFSTR("ship_right"));
	fStationaryImage = SRGraphicsUtils::CreateCGImageForResourceNamed(CFSTR("ship_normal"));
	
	fImage = fStationaryImage;
}

SRPlayerSprite::~SRPlayerSprite()
{
    // get rid of the ship's icons
	CGImageRelease(fLeftImage);
	CGImageRelease(fRightImage);
	CGImageRelease(fStationaryImage);
    
    fInstance = 0;
}

void SRPlayerSprite::ResetPosition()
{
    // put the ship in the middle of the bottom of the screen...
    fX = kGameWindowWidth / 2;
    fY = kGameStatusBarHeight + kBoundary;
    fVelocityX = 0;
    fVelocityY = 0;
    fDirection = DirectionHalt;
}

void SRPlayerSprite::TakeCommand( Command cmd )
{
    // if the command is a fire, we fire, otherwise it's a move command
    switch(cmd)
    {
        case WeaponFire:
            fFireNow = true;
            break;
            
        default:
            fDirection = cmd;
            break;
    }
}

void SRPlayerSprite::Think()
{
    // If we can fire, and we want to fire, then do it!
    if(fAmmoTimer > 0) fAmmoTimer--;

    if( fFireNow && (fAmmoTimer == 0) )
    {
        SRAmmoSprite *ammo = new SRAmmoSprite( CFSTR("ship_ammo"), this );
        
        ammo->setX( fX );
        ammo->setY( fY + kAmmoDistance );
        
        ammo->pushY( kAmmoForce );
        
        fFireNow = false;
        fAmmoTimer = kAmmoCountdown;
        
        SRSoundManager::PlaySoundEffect( SRSoundManager::SFXplayerFires );
    }
    
    // Have we passed a boundary?  If so, let's stop!
    if( fX <= kBoundary )
    {
        fDirection = DirectionHalt;
        fX = kBoundary + 1;
    }
    else if( fX >= (kGameWindowWidth - kBoundary) )
    {
        fDirection = DirectionHalt;
        fX = (kGameWindowWidth - (kBoundary + 1));
    }
    
    if( fY < (kGameStatusBarHeight + kBoundary) )
    {
        fY = (kGameStatusBarHeight + kBoundary);
        fVelocityY = 0;
    }
    else if( fY > (kGameStatusBarHeight + kBoundary + kZone) )
    {
        fY = (kGameStatusBarHeight + kBoundary + kZone);
    }
    
    // Depending on the direction, we choose a speed
    switch(fDirection)
    {
        case DirectionHalt:
            fVelocityX = 0;
            fVelocityY = 0;
            break;
            
        case DirectionLeft:
            fVelocityX = -2;
            break;
            
        case DirectionRight:
            fVelocityX = 2;
            break;
        
        case DirectionUp:
            fVelocityY = 2;
            break;
        
        case DirectionDown:
            fVelocityY = -2;
            break;
		
		default:
			// Do nothing, in this case
			break;
    }
    
    // give feedback as to which direction we're flying in
    if(fVelocityX < 0)
        fImage = fLeftImage;
    else if(fVelocityX > 0)
        fImage = fRightImage;
    else
        fImage = fStationaryImage;

	SRCgImageSprite::Think();
}

// one of the bullets hit something solid!
void SRPlayerSprite::ChildDied( SRObject * )
{
}

// something hit us!
bool SRPlayerSprite::Hit( SRObject *other )
{
#if DEBUG
	// In debug mode, no one can hear you scream!
	//return false;
#endif
    if( other->Type() == kEnemyAmmoType || other->Type() == kGenericAmmoType )
    {
        SRSoundManager::PlaySoundEffect( SRSoundManager::SFXplayerHit );
        SRGameObject::instance->PlayerDied();
		// TODO: It'd be nice to have a small period of time
		// after dying where the player is invulnerable, like
		// other games.
        return true;
    }
    
    return false;
}
