#include "SRStar.h"
#include "WindowDefs.h"
#include "SRGameObject.h"
#include "SRGraphicsUtils.h"

SRStar::SRStar( float x, float y, RGBColor starColor )
	: fStarColor(starColor)
{
    setX(x);
    setY(y);
}

SRStar::~SRStar()
{
}
    
void SRStar::Draw( CGContextRef theRef )
{
	if( SRGraphicsUtils::UsingQuickdraw || theRef == 0 )
	{
		// This would be an error
		abort();
	
		RGBColor theColor = fStarColor;
		RGBForeColor(&theColor);
		
		if(fVelocityY != kMaxStarSpeed) PenSize( 2, 1 );
		else PenSize( 1, 1 );

		MoveTo(fX, fY);
		::Line(0, 1);
	}
	else
	{
		CGContextSetRGBStrokeColor( theRef, fStarColor.red(),
									fStarColor.green(),
									fStarColor.blue(),
									fStarColor.alpha() );
		if( fVelocityY < kMaxStarSpeed )
		{
			CGContextSetLineWidth( theRef, 2.0 );
		}
		else
		{
			CGContextSetLineWidth( theRef, 1.0 );
		}
		
		// One of the many reasons to switch to CoreGraphics: round end-caps!
		CGContextSetLineCap( theRef, kCGLineCapRound );
		
		CGContextBeginPath(theRef);
		
		CGContextMoveToPoint( theRef, fX, fY );
		CGContextAddLineToPoint( theRef, fX, fY + 1 );
		
		CGContextStrokePath(theRef);
		
		CGContextClosePath(theRef);
	}
}

void SRStar::Think()
{
	if( fX < 0.0f )
	{
		fX += kGameWindowWidth;
	}
	else if( fX > kGameWindowWidth )
	{
		fX -= kGameWindowWidth;
	}

	if( fY < kGameStatusBarHeight )
	{
		fY += kGameWindowHeight;
	}
	else if( fY > (kGameWindowHeight + kGameStatusBarHeight) )
	{
		fY -= kGameWindowHeight;
	}
}
