/***********************************************************************************
  TODO: 
         levels
         bonus goodies
         high score list
         new about box
***********************************************************************************/


#define IN_MAIN

#ifdef __APPLE__
#include <Carbon/Carbon.h>
#else
#include <Carbon.h>
#endif

#include <CoreFoundation/CFPreferences.h>

#include "main.h"
#include "WindowDefs.h"
#include "Handlers.h"
#include "SRSoundManager.h"
#include "SRGameObject.h"

int gGameWindowX;
int gGameWindowY;
Handle gMenuBar;
CFStringRef prefX = CFSTR( "GameWindowX" );
CFStringRef prefY = CFSTR( "GameWindowY" );
CFStringRef prefMus = CFSTR( "MusicOn" );
CFStringRef prefFX = CFSTR( "FXOn" );
CFStringRef appID = CFSTR( "ca.skwirl.spacerat" );

int main( int /*argc*/, char **/*argv*/ )
{
    Init();
    
    Begin();
    
    Takedown();
    
    return 0;
}

void Init()
{
    InitCursor();
    
    OSErr err;
    
    // install quit handler
    err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP(QuitAppleEventHandler), 0, false);
	if (err != noErr)
	{
		DieWithError();
    }

    // set the random seed
    UnsignedWide microseconds;
    Microseconds( &microseconds );
    srandom(microseconds.lo);

    // load up menus
    LoadMenus();
    
    bool success = CFPreferencesAppSynchronize( appID );

    SRSoundManager::InitSound();
    
    bool isValid, musicOn, FXOn;
    
    musicOn = CFPreferencesGetAppBooleanValue( prefMus, appID, (unsigned char *)&isValid );
    
    if(isValid)
    {
        if(SRSoundManager::IsPlaying() ^ musicOn)
            SRSoundManager::TogglePlaying();
    }

    FXOn = CFPreferencesGetAppBooleanValue( prefFX, appID, (unsigned char *)&isValid );
    
    if(isValid)
    {
        if(SRSoundManager::AreFXOn() ^ FXOn)
            SRSoundManager::ToggleFX();
    }
}

void LoadMenus()
{
	//MenuRef 	menu;
	long 		response;
	OSStatus	err = noErr;

    // get the game's menu bar
	gMenuBar = GetNewMBar(128);
	if (gMenuBar)
	{
	    // set it as the active one
		SetMenuBar(gMenuBar);

		// see if we need to comply with the Aqua HI guidelines
		err = Gestalt(gestaltMenuMgrAttr, &response);
		if ((err == noErr) && (response & gestaltMenuMgrAquaLayoutMask))
		{
		    // if so, then we shouldn't use a quit item, it'll be in the app menu
		    // so since that's all that's in the File menu, we dump the file menu
			//menu = GetMenuHandle(129);
			DeleteMenu(129);
		}
		DrawMenuBar();
	}
	else
		DieWithError();

}

void CreateGameWindow()
{
    int x = 50;
    int y = 50;
    Rect            windowRect;
    OSStatus        err;
    
    // get the game window preference
    bool isValid;
    gGameWindowX = CFPreferencesGetAppIntegerValue( prefX, appID, 
                                                    (unsigned char *)&isValid);
    if(isValid) x = gGameWindowX;
    else gGameWindowX = x;
    	
    gGameWindowY = CFPreferencesGetAppIntegerValue( prefY, appID, 
                                                    (unsigned char *)&isValid);
    if(isValid) y = gGameWindowY;
    else gGameWindowY = y;
    	
    // Create our game window up in the top left
    SetRect( &windowRect, x, y, x+kGameWindowWidth, y+kGameWindowHeight+kGameStatusBarHeight );

    err = CreateNewWindow( kMovableModalWindowClass, kWindowStandardHandlerAttribute, 
                           &windowRect, &gGameWindow );
    SetWTitle( gGameWindow, "\pSpaceRat" );
}

void Begin()
{
    // create the game object
    new SRGameObject();
    
    // create the game window
    CreateGameWindow();

    // install the event handlers, for keypresses, window, etc
    InstallEventHandlers();

    // show the window
    ShowWindow(gGameWindow);

    // Let's get this show on the road!
    RunApplicationEventLoop();
}

void SavePrefs()
{
    // Set preferences
    Rect bounds;
    
    GetWindowBounds( gGameWindow, kWindowContentRgn, &bounds );
    gGameWindowX = bounds.left;
    gGameWindowY = bounds.top;
    
    CFNumberRef x = CFNumberCreate( NULL, kCFNumberIntType, &gGameWindowX );
    CFNumberRef y = CFNumberCreate( NULL, kCFNumberIntType, &gGameWindowY );
    
    CFPreferencesSetAppValue( prefX, x, appID );
    CFPreferencesSetAppValue( prefY, y, appID );
    
    CFRelease(x);
    CFRelease(y);
    
    bool musicOn = SRSoundManager::IsPlaying();
    
    CFPreferencesSetAppValue( prefMus, musicOn ? kCFBooleanTrue : kCFBooleanFalse,
                              appID );
    
    bool FXOn = SRSoundManager::AreFXOn();
    
    CFPreferencesSetAppValue( prefFX, FXOn ? kCFBooleanTrue : kCFBooleanFalse,
                              appID );
    
    bool success = CFPreferencesAppSynchronize( appID );
}

void Takedown()
{
    SavePrefs();
    SRSoundManager::EndSound();
    DisposeWindow(gGameWindow);
}

void DieWithError()
{
    Alert( kErrorAlert, nil );
    ExitToShell();
}
